@@ -6,6 +6,7 @@ import (
 	"reflect"
 
 	"github.com/openshift/elasticsearch-operator/pkg/utils"
+	"github.com/openshift/elasticsearch-operator/pkg/utils/comparators"
 	"github.com/sirupsen/logrus"
 	v1 "k8s.io/api/core/v1"
 	"k8s.io/apimachinery/pkg/types"
@@ -14,9 +15,13 @@ import (
 	api "github.com/openshift/elasticsearch-operator/pkg/apis/logging/v1"
 )
 
+const expectedMinVersion = "6.0"
+
 var wrongConfig bool
 var nodes map[string][]NodeTypeInterface
 
+var aliasNeededMap map[string]bool
+
 func FlushNodes(clusterName, namespace string) {
 	nodes[nodeMapKey(clusterName, namespace)] = []NodeTypeInterface{}
 }
@@ -27,6 +32,7 @@ func nodeMapKey(clusterName, namespace string) string {
 
 // CreateOrUpdateElasticsearchCluster creates an Elasticsearch deployment
 func (elasticsearchRequest *ElasticsearchRequest) CreateOrUpdateElasticsearchCluster() error {
+	esClient := elasticsearchRequest.esClient
 
 	// Verify that we didn't scale up too many masters
 	err := elasticsearchRequest.isValidConf()
@@ -45,12 +51,11 @@ func (elasticsearchRequest *ElasticsearchRequest) CreateOrUpdateElasticsearchClu
 
 	//clearing transient setting because of a bug in earlier releases which
 	//may leave the shard allocation in an undesirable state
-	cluster := elasticsearchRequest.cluster
-	if success, err := ClearTransientShardAllocation(cluster.Name, cluster.Namespace, elasticsearchRequest.client); !success {
+	if success, err := esClient.ClearTransientShardAllocation(); !success {
 		logrus.Warnf("Unable to clear transient shard allocation for %q %q: %v", elasticsearchRequest.cluster.Namespace, elasticsearchRequest.cluster.Namespace, err)
 	}
 	progressUnshedulableNodes(elasticsearchRequest.cluster)
-	err = elasticsearchRequest.performFullClusterRestart()
+	err = elasticsearchRequest.performCertClusterRestart()
 	if err != nil {
 		return elasticsearchRequest.UpdateClusterStatus()
 	}
@@ -65,30 +70,55 @@ func (elasticsearchRequest *ElasticsearchRequest) CreateOrUpdateElasticsearchClu
 
 		if _, ok := containsNodeTypeInterface(upgradeInProgressNode, scheduledUpgradeNodes); ok {
 			logrus.Debugf("Continuing update for %v", upgradeInProgressNode.name())
-			upgradeInProgressNode.update(nodeStatus)
+			if err := upgradeInProgressNode.update(nodeStatus); err != nil {
+				logrus.Errorf("unable to update node. E: %s", err.Error())
+			}
 		} else {
 			logrus.Debugf("Continuing restart for %v", upgradeInProgressNode.name())
 			upgradeInProgressNode.rollingRestart(nodeStatus)
 		}
 
 		addNodeState(upgradeInProgressNode, nodeStatus)
-		elasticsearchRequest.setNodeStatus(upgradeInProgressNode, nodeStatus, clusterStatus)
+		if err := elasticsearchRequest.setNodeStatus(upgradeInProgressNode, nodeStatus, clusterStatus); err != nil {
+			logrus.Errorf("unable to update node status: E: %s", err)
+		}
 
 	} else {
 
 		if len(scheduledUpgradeNodes) > 0 {
-			for _, node := range scheduledUpgradeNodes {
-				logrus.Debugf("Perform a update for %v", node.name())
-				clusterStatus := elasticsearchRequest.cluster.Status.DeepCopy()
-				_, nodeStatus := getNodeStatus(node.name(), clusterStatus)
 
-				err := node.update(nodeStatus)
+			// get the current ES version
+			version, err := esClient.GetLowestClusterVersion()
+			if err != nil {
+				// this can be because we couldn't get a valid response from ES
+				logrus.Warnf("when trying to get LowestClusterVersion: %v", err)
+			} else {
 
-				addNodeState(node, nodeStatus)
-				elasticsearchRequest.setNodeStatus(node, nodeStatus, clusterStatus)
+				logrus.Debugf("Found current cluster version to be %q", version)
+				comparison := comparators.CompareVersions(version, expectedMinVersion)
 
-				if err != nil {
-					logrus.Warnf("Error occurred while updating node %v: %v", node.name(), err)
+				// if it is < what we expect (6.0) then do full cluster restart:
+				if comparison > 0 {
+					// perform a full cluster restart
+					if err := elasticsearchRequest.fullClusterRestart(scheduledUpgradeNodes); err != nil {
+						logrus.Warnf("when trying to perform full cluster restart: %v", err)
+					}
+
+				} else {
+
+					for _, node := range scheduledUpgradeNodes {
+						logrus.Debugf("Perform a update for %v", node.name())
+						clusterStatus := elasticsearchRequest.cluster.Status.DeepCopy()
+						_, nodeStatus := getNodeStatus(node.name(), clusterStatus)
+						err := node.update(nodeStatus)
+						addNodeState(node, nodeStatus)
+						if err := elasticsearchRequest.setNodeStatus(node, nodeStatus, clusterStatus); err != nil {
+							logrus.Error(err)
+						}
+						if err != nil {
+							logrus.Warnf("Error occurred while updating node %v: %v", node.name(), err)
+						}
+					}
 				}
 			}
 
@@ -106,7 +136,9 @@ func (elasticsearchRequest *ElasticsearchRequest) CreateOrUpdateElasticsearchClu
 					node.rollingRestart(nodeStatus)
 
 					addNodeState(node, nodeStatus)
-					elasticsearchRequest.setNodeStatus(node, nodeStatus, clusterStatus)
+					if err := elasticsearchRequest.setNodeStatus(node, nodeStatus, clusterStatus); err != nil {
+						logrus.Errorf("unable to set node status. E: %s", err.Error())
+					}
 				}
 
 			} else {
@@ -131,22 +163,35 @@ func (elasticsearchRequest *ElasticsearchRequest) CreateOrUpdateElasticsearchClu
 					}
 
 					addNodeState(node, nodeStatus)
-					elasticsearchRequest.setNodeStatus(node, nodeStatus, clusterStatus)
+					if err := elasticsearchRequest.setNodeStatus(node, nodeStatus, clusterStatus); err != nil {
+						logrus.Errorf("unable to set node status. E: %s", err.Error())
+					}
 
 					elasticsearchRequest.updateMinMasters()
 				}
 
-				// reenable shard allocation
-				if ok, err := SetShardAllocation(cluster.Name, cluster.Namespace, api.ShardAllocationAll, elasticsearchRequest.client); !ok {
+				// ensure we always have shard allocation to All if we aren't doing an upgrade...
+				if ok, err := esClient.SetShardAllocation(api.ShardAllocationAll); !ok {
 					logrus.Warnf("Unable to enable shard allocation: %v", err)
 				}
 
 				// we only want to update our replicas if we aren't in the middle up an upgrade
-				UpdateReplicaCount(
-					elasticsearchRequest.cluster.Name,
-					elasticsearchRequest.cluster.Namespace,
-					elasticsearchRequest.client,
-					int32(calculateReplicaCount(elasticsearchRequest.cluster)))
+				if err := esClient.UpdateReplicaCount(int32(calculateReplicaCount(elasticsearchRequest.cluster))); err != nil {
+					logrus.Error(err)
+				}
+				if aliasNeededMap == nil {
+					aliasNeededMap = make(map[string]bool)
+				}
+				if val, ok := aliasNeededMap[nodeMapKey(elasticsearchRequest.cluster.Name, elasticsearchRequest.cluster.Namespace)]; !ok || val {
+					// add alias to old indices if they exist and don't have one
+					// this should be removed after one release...
+					successful := esClient.AddAliasForOldIndices()
+
+					if successful {
+						aliasNeededMap[nodeMapKey(elasticsearchRequest.cluster.Name, elasticsearchRequest.cluster.Namespace)] = false
+					}
+
+				}
 			}
 		}
 	}
@@ -159,19 +204,23 @@ func (elasticsearchRequest *ElasticsearchRequest) updateMinMasters() {
 	// do as best effort -- whenever we create a node update min masters (if required)
 
 	cluster := elasticsearchRequest.cluster
+	esClient := elasticsearchRequest.esClient
 
-	currentMasterCount, err := GetMinMasterNodes(cluster.Name, cluster.Namespace, elasticsearchRequest.client)
+	currentMasterCount, err := esClient.GetMinMasterNodes()
 	if err != nil {
 		logrus.Debugf("Unable to get current min master count for cluster %v", cluster.Name)
 	}
 
 	desiredMasterCount := getMasterCount(cluster)/2 + 1
-	currentNodeCount, err := GetClusterNodeCount(cluster.Name, cluster.Namespace, elasticsearchRequest.client)
+	currentNodeCount, err := esClient.GetClusterNodeCount()
+	if err != nil {
+		logrus.Warnf("unable to get cluster node count. E: %s\r\n", err.Error())
+	}
 
 	// check that we have the required number of master nodes in the cluster...
 	if currentNodeCount >= desiredMasterCount {
 		if currentMasterCount != desiredMasterCount {
-			if _, setErr := SetMinMasterNodes(cluster.Name, cluster.Namespace, desiredMasterCount, elasticsearchRequest.client); setErr != nil {
+			if _, setErr := esClient.SetMinMasterNodes(desiredMasterCount); setErr != nil {
 				logrus.Debugf("Unable to set min master count to %d for cluster %v", desiredMasterCount, cluster.Name)
 			}
 		}
@@ -292,7 +341,9 @@ func (elasticsearchRequest *ElasticsearchRequest) getNodes() {
 				elasticsearchRequest.updateMinMasters()
 				minMasterUpdated = true
 			}
-			node.delete()
+			if err := node.delete(); err != nil {
+				logrus.Warnf("unable to delete node. E: %s\r\n", err.Error())
+			}
 
 			// remove from status.Nodes
 			if index, _ := getNodeStatus(node.name(), &cluster.Status); index != NOT_FOUND_INDEX {
@@ -374,7 +425,7 @@ func addNodeState(node NodeTypeInterface, nodeStatus *api.ElasticsearchNodeStatu
 	nodeStatus.StatefulSetName = nodeState.StatefulSetName
 }
 
-func (elasticsearchRequest *ElasticsearchRequest) setNodeStatus(node NodeTypeInterface, nodeStatus *api.ElasticsearchNodeStatus, clusterStatus *api.ElasticsearchStatus) {
+func (elasticsearchRequest *ElasticsearchRequest) setNodeStatus(node NodeTypeInterface, nodeStatus *api.ElasticsearchNodeStatus, clusterStatus *api.ElasticsearchStatus) error {
 
 	index, _ := getNodeStatus(node.name(), clusterStatus)
 
@@ -384,7 +435,7 @@ func (elasticsearchRequest *ElasticsearchRequest) setNodeStatus(node NodeTypeInt
 		clusterStatus.Nodes[index] = *nodeStatus
 	}
 
-	elasticsearchRequest.updateNodeStatus(*clusterStatus)
+	return elasticsearchRequest.updateNodeStatus(*clusterStatus)
 }
 
 func (elasticsearchRequest *ElasticsearchRequest) updateNodeStatus(status api.ElasticsearchStatus) error {
@@ -421,10 +472,11 @@ func (elasticsearchRequest *ElasticsearchRequest) updateNodeStatus(status api.El
 	return nil
 }
 
-// Full cluster restart is required when certs need to be refreshed
+// performCertClusterRestart is required when certs need to be refreshed
 // this is a very high priority action since the cluster may be fractured/unusable
 // in the case where certs aren't all rolled out correctly or are expired
-func (elasticsearchRequest *ElasticsearchRequest) performFullClusterRestart() error {
+func (elasticsearchRequest *ElasticsearchRequest) performCertClusterRestart() error {
+	esClient := elasticsearchRequest.esClient
 
 	// make sure we have nodes that are scheduled for full cluster restart first
 	certRedeployNodes := getScheduledCertRedeployNodes(elasticsearchRequest.cluster)
@@ -451,12 +503,12 @@ func (elasticsearchRequest *ElasticsearchRequest) performFullClusterRestart() er
 			containsClusterCondition(api.UpdatingSettings, v1.ConditionTrue, clusterStatus) {
 
 			// disable shard allocation
-			if ok, err := SetShardAllocation(elasticsearchRequest.cluster.Name, elasticsearchRequest.cluster.Namespace, api.ShardAllocationPrimaries, elasticsearchRequest.client); !ok {
+			if ok, err := esClient.SetShardAllocation(api.ShardAllocationPrimaries); !ok {
 				logrus.Warnf("Unable to set shard allocation to primaries: %v", err)
 			}
 
 			// flush nodes
-			if ok, err := DoSynchronizedFlush(elasticsearchRequest.cluster.Name, elasticsearchRequest.cluster.Namespace, elasticsearchRequest.client); !ok {
+			if ok, err := esClient.DoSynchronizedFlush(); !ok {
 				logrus.Warnf("Unable to perform synchronized flush: %v", err)
 			}
 
@@ -474,7 +526,9 @@ func (elasticsearchRequest *ElasticsearchRequest) performFullClusterRestart() er
 				_, nodeStatus := getNodeStatus(node.name(), clusterStatus)
 				node.fullClusterRestart(nodeStatus)
 				addNodeState(node, nodeStatus)
-				elasticsearchRequest.setNodeStatus(node, nodeStatus, clusterStatus)
+				if err := elasticsearchRequest.setNodeStatus(node, nodeStatus, clusterStatus); err != nil {
+					logrus.Warnf("unable to set node status. %s", err.Error())
+				}
 			}
 
 			// check that all nodes have been restarted by seeing if they still have the need to cert restart
@@ -502,7 +556,7 @@ func (elasticsearchRequest *ElasticsearchRequest) performFullClusterRestart() er
 		}
 
 		// reenable shard allocation
-		if ok, err := SetShardAllocation(elasticsearchRequest.cluster.Name, elasticsearchRequest.cluster.Namespace, api.ShardAllocationAll, elasticsearchRequest.client); !ok {
+		if ok, err := esClient.SetShardAllocation(api.ShardAllocationAll); !ok {
 			logrus.Warnf("Unable to enable shard allocation: %v", err)
 			return err
 		}
@@ -513,9 +567,13 @@ func (elasticsearchRequest *ElasticsearchRequest) performFullClusterRestart() er
 	// 5 -- recovery
 	// wait for cluster to go green again
 	if containsClusterCondition(api.Restarting, v1.ConditionTrue, clusterStatus) {
-		if status, _ := GetClusterHealthStatus(elasticsearchRequest.cluster.Name, elasticsearchRequest.cluster.Namespace, elasticsearchRequest.client); status != "green" {
-			logrus.Infof("Waiting for cluster to complete recovery: %v / green", status)
-			return fmt.Errorf("Cluster has not completed recovery after restart: %v / green", status)
+		status, err := esClient.GetClusterHealthStatus()
+		if err != nil {
+			return fmt.Errorf("Receieved error while trying to get cluster health for %q: %v", elasticsearchRequest.cluster.Name, err)
+		}
+
+		if !utils.Contains(desiredClusterStates, status) {
+			return fmt.Errorf("Waiting for cluster to recover: %s / %v", status, desiredClusterStates)
 		}
 
 		logrus.Infof("Completed full cluster restart for cert redeploy on %v", elasticsearchRequest.cluster.Name)
@@ -524,3 +582,112 @@ func (elasticsearchRequest *ElasticsearchRequest) performFullClusterRestart() er
 
 	return nil
 }
+
+func (elasticsearchRequest *ElasticsearchRequest) fullClusterRestart(scheduledUpgradeNodes []NodeTypeInterface) error {
+	esClient := elasticsearchRequest.esClient
+	clusterStatus := &elasticsearchRequest.cluster.Status
+
+	// 1 -- precheck
+	// no restarting conditions set
+	if len(scheduledUpgradeNodes) > 0 {
+
+		if containsClusterCondition(api.Restarting, v1.ConditionFalse, clusterStatus) &&
+			containsClusterCondition(api.UpdatingSettings, v1.ConditionFalse, clusterStatus) {
+
+			if status, _ := esClient.GetClusterHealthStatus(); !utils.Contains(desiredClusterStates, status) {
+				return fmt.Errorf("Waiting for cluster %q to be recovered before restarting: %s / %v", elasticsearchRequest.cluster.Name, status, desiredClusterStates)
+			}
+
+			logrus.Infof("Beginning full cluster restart on %v", elasticsearchRequest.cluster.Name)
+
+			// set conditions here for next check
+			updateUpdatingSettingsCondition(clusterStatus, v1.ConditionTrue)
+		}
+
+		// 2 -- prep for restart
+		// condition updatingsettings true
+		if containsClusterCondition(api.Restarting, v1.ConditionFalse, clusterStatus) &&
+			containsClusterCondition(api.UpdatingSettings, v1.ConditionTrue, clusterStatus) {
+
+			// disable shard allocation
+			if ok, err := esClient.SetShardAllocation(api.ShardAllocationPrimaries); !ok {
+				return fmt.Errorf("Unable to set shard allocation to primaries: %v", err)
+			}
+
+			// flush nodes
+			if ok, err := esClient.DoSynchronizedFlush(); !ok {
+				logrus.Warnf("Unable to perform synchronized flush: %v", err)
+			}
+
+			updateRestartingCondition(clusterStatus, v1.ConditionTrue)
+			updateUpdatingSettingsCondition(clusterStatus, v1.ConditionFalse)
+		}
+
+		// 3 -- restart
+		// condition restarting true
+		if containsClusterCondition(api.Restarting, v1.ConditionTrue, clusterStatus) &&
+			containsClusterCondition(api.UpdatingSettings, v1.ConditionFalse, clusterStatus) {
+
+			// call fullClusterRestart on each node that is scheduled for a full cluster restart
+			for _, node := range scheduledUpgradeNodes {
+				_, nodeStatus := getNodeStatus(node.name(), clusterStatus)
+
+				node.progressNodeChanges(nodeStatus)
+
+				addNodeState(node, nodeStatus)
+				if err := elasticsearchRequest.setNodeStatus(node, nodeStatus, clusterStatus); err != nil {
+					logrus.Warnf("unable to set node status. %s", err.Error())
+				}
+			}
+
+			// check that all nodes have been restarted by seeing if they still have the need to cert restart
+			if len(getScheduledUpgradeNodes(elasticsearchRequest.cluster)) > 0 {
+				return fmt.Errorf("Not all nodes were able to be restarted yet")
+			}
+
+			updateUpdatingSettingsCondition(clusterStatus, v1.ConditionTrue)
+		}
+	}
+
+	// 4 -- post restart
+	// condition restarting true and updatingsettings true
+	if containsClusterCondition(api.Restarting, v1.ConditionTrue, clusterStatus) &&
+		containsClusterCondition(api.UpdatingSettings, v1.ConditionTrue, clusterStatus) {
+
+		// verify all nodes rejoined
+		// check that we have no failed/notReady nodes
+		logrus.Infof("Waiting for all nodes to rejoin cluster %q in namespace %q", elasticsearchRequest.cluster.Name, elasticsearchRequest.cluster.Namespace)
+
+		for _, node := range scheduledUpgradeNodes {
+			if err, _ := node.waitForNodeRejoinCluster(); err != nil {
+				return fmt.Errorf("Timed out waiting for %v to rejoin cluster %v", node.name(), elasticsearchRequest.cluster.Name)
+			}
+		}
+
+		// reenable shard allocation
+		if ok, err := esClient.SetShardAllocation(api.ShardAllocationAll); !ok {
+			return fmt.Errorf("Unable to enable shard allocation: %v", err)
+		}
+
+		updateUpdatingSettingsCondition(clusterStatus, v1.ConditionFalse)
+	}
+
+	// 5 -- recovery
+	// wait for cluster to go yellow/green again
+	if containsClusterCondition(api.Restarting, v1.ConditionTrue, clusterStatus) {
+
+		status, err := esClient.GetClusterHealthStatus()
+		if err != nil {
+			return fmt.Errorf("Receieved error while trying to get cluster health for %q: %v", elasticsearchRequest.cluster.Name, err)
+		}
+
+		if !utils.Contains(desiredClusterStates, status) {
+			return fmt.Errorf("Waiting for cluster to recover: %s / %v", status, desiredClusterStates)
+		}
+
+		logrus.Infof("Completed full cluster restart on %v", elasticsearchRequest.cluster.Name)
+		updateRestartingCondition(clusterStatus, v1.ConditionFalse)
+	}
+
+	return nil
+}
