@@ -9,13 +9,11 @@ import (
 const (
 	modeUnique    = "unique"
 	modeSharedOps = "shared_ops"
-	defaultMode   = modeSharedOps
 
-	defaultMasterCPURequest   = "100m"
-	defaultCPURequest         = "100m"
-	defaultMemoryLimit        = "4Gi"
-	defaultMemoryRequest      = "1Gi"
-	elasticsearchDefaultImage = "quay.io/openshift/origin-logging-elasticsearch5"
+	defaultMode          = modeSharedOps
+	defaultCPURequest    = "100m"
+	defaultMemoryLimit   = "4Gi"
+	defaultMemoryRequest = "1Gi"
 
 	maxMasterCount = 3
 
@@ -25,9 +23,8 @@ const (
 
 	k8sTokenFile = "/var/run/secrets/kubernetes.io/serviceaccount/token"
 
-	logAppenderAnnotation = "elasticsearch.openshift.io/develLogAppender"
-	yellowClusterState    = "yellow"
-	greenClusterState     = "green"
+	yellowClusterState = "yellow"
+	greenClusterState  = "green"
 )
 
 var desiredClusterStates = []string{yellowClusterState, greenClusterState}
@@ -46,13 +43,6 @@ func esUnicastHost(clusterName, namespace string) string {
 	return fmt.Sprintf("%v-cluster.%v.svc", clusterName, namespace)
 }
 
-func rootLogger(cluster *api.Elasticsearch) string {
-	if value, ok := cluster.GetAnnotations()[log4jConfig]; ok {
-		return value
-	}
-	return "rolling"
-}
-
 func calculateReplicaCount(dpl *api.Elasticsearch) int {
 	dataNodeCount := int((getDataCount(dpl)))
 	repType := dpl.Spec.RedundancyPolicy
