@@ -14,7 +14,7 @@ import (
 	"k8s.io/apimachinery/pkg/util/wait"
 	"k8s.io/client-go/kubernetes"
 
-	"github.com/openshift/elasticsearch-operator/pkg/k8shandler"
+	"github.com/openshift/elasticsearch-operator/pkg/elasticsearch"
 	"github.com/openshift/elasticsearch-operator/pkg/utils"
 	"sigs.k8s.io/controller-runtime/pkg/client/fake"
 
@@ -36,6 +36,17 @@ func GetFileContents(filePath string) []byte {
 	return contents
 }
 
+func ConfigMap(name, namespace string, labels, data map[string]string) *v1.ConfigMap {
+	return &v1.ConfigMap{
+		ObjectMeta: metav1.ObjectMeta{
+			Name:      name,
+			Namespace: namespace,
+			Labels:    labels,
+		},
+		Data: data,
+	}
+}
+
 func Secret(secretName string, namespace string, data map[string][]byte) *v1.Secret {
 	return &v1.Secret{
 		TypeMeta: metav1.TypeMeta{
@@ -66,10 +77,15 @@ func WaitForNodeStatusCondition(t *testing.T, f *framework.Framework, namespace,
 		}
 
 		allMatch := true
-
 		for _, node := range elasticsearchCR.Status.Nodes {
+			t.Log("\tActual  status", node.UpgradeStatus)
+			t.Log("\tDesired status", condition)
 			if !reflect.DeepEqual(node.UpgradeStatus, condition) {
+				t.Log("\t\tDid not match")
 				allMatch = false
+			} else {
+				t.Log("\t\tMatch!")
+				break
 			}
 		}
 
@@ -101,8 +117,9 @@ func WaitForClusterStatusCondition(t *testing.T, f *framework.Framework, namespa
 		}
 
 		contained := false
-
 		for _, clusterCondition := range elasticsearchCR.Status.Conditions {
+			t.Log("\tExpected condition", condition)
+			t.Log("\tReal     condition", clusterCondition)
 			if reflect.DeepEqual(clusterCondition, condition) {
 				contained = true
 			}
@@ -129,19 +146,16 @@ func WaitForStatefulset(t *testing.T, kubeclient kubernetes.Interface, namespace
 				t.Logf("Waiting for availability of %s statefulset\n", name)
 				return false, nil
 			}
-			t.Logf("Error getting statefull set: %s\n", err.Error())
 			return false, err
 		}
 
 		if int(statefulset.Status.ReadyReplicas) == replicas {
-			t.Logf("Amount of replicas are equal to expected")
 			return true, nil
 		}
 		t.Logf("Waiting for full availability of %s statefulset (%d/%d)\n", name, statefulset.Status.ReadyReplicas, replicas)
 		return false, nil
 	})
 	if err != nil {
-		t.Logf("Poll error: %s\n", err.Error())
 		return err
 	}
 	t.Logf("Statefulset available (%d/%d)\n", replicas, replicas)
@@ -161,10 +175,11 @@ func GenerateUUID() string {
 func WaitForIndexTemplateReplicas(t *testing.T, kubeclient kubernetes.Interface, namespace, clusterName string, replicas int32, retryInterval, timeout time.Duration) error {
 	// mock out Secret response from client
 	mockClient := fake.NewFakeClient(getMockedSecret(clusterName, namespace))
+	esClient := elasticsearch.NewClient(clusterName, namespace, mockClient)
 
 	err := wait.Poll(retryInterval, timeout, func() (done bool, err error) {
 		// get all index replica count
-		indexTemplates, err := k8shandler.GetIndexTemplates(clusterName, namespace, mockClient)
+		indexTemplates, err := esClient.GetIndexTemplates()
 		if err != nil {
 			t.Logf("Received error: %v", err)
 			return false, nil
@@ -201,11 +216,12 @@ func WaitForIndexTemplateReplicas(t *testing.T, kubeclient kubernetes.Interface,
 func WaitForIndexReplicas(t *testing.T, kubeclient kubernetes.Interface, namespace, clusterName string, replicas int32, retryInterval, timeout time.Duration) error {
 	// mock out Secret response from client
 	mockClient := fake.NewFakeClient(getMockedSecret(clusterName, namespace))
+	esClient := elasticsearch.NewClient(clusterName, namespace, mockClient)
 
 	err := wait.Poll(retryInterval, timeout, func() (done bool, err error) {
 
 		// get all index replica count
-		indexHealth, err := k8shandler.GetIndexReplicaCounts(clusterName, namespace, mockClient)
+		indexHealth, err := esClient.GetIndexReplicaCounts()
 		if err != nil {
 			return false, nil
 		}
